﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
    
// Дополнительные пространства имен
using System.Data.OleDb;
using System.Data.Common;
    
namespace WinForms2
{
    public partial class Form1 : Form
    {
        // Инфраструктура для интерфейса первой вкладки
        TabControl tabControl;
        ComboBox cbLetters;
        Button button;
        ListBox listBox;
    
        // Инфраструктура для доступа к БД
        OleDbConnection connection;
        OleDbDataAdapter adapter;
        DataTable tableCustomers;
    
        public Form1()
        {
            InitializeComponent();
            this.Text = "Упражнение 2. Построчное редактирование";
            this.Width = 355;
            this.FormClosed += Form1_FormClosed;
    
            // Добавили панель к форме
            this.Controls.Add(tabControl = new TabControl());
            tabControl.Dock = DockStyle.Fill;
    
            // Создать интерфейс пользователя
            CreatePage1();
            CreatePage2();
    
            // Заполнить cbLetters, полную таблицу и связать с listBox
            LoadPage1();
        }
    
        void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Выполнена ли функция tableCustomers.AcceptChanges(),
            // меняющая состояние таблицы при обновлении в this.Update() 
            DataTable table = tableCustomers.GetChanges();
            if (table != null)
            {
                DialogResult result =
                    MessageBox.Show("Вы не сохранили изменения в БД\nСохранить?",
                    "Внимание!",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                if (result == DialogResult.Yes)
                    Update();
            }
        }
    
        // Строка соединения к БД с абсолютным путем, определяемым сборкой
        String ConnectionString()
        {
            // Используем построитель строки подключения 
            OleDbConnectionStringBuilder objConnectionStringBuilder =
                new OleDbConnectionStringBuilder();
            objConnectionStringBuilder.Provider = "Microsoft.Jet.OLEDB.4.0";
            objConnectionStringBuilder.DataSource =
                Application.StartupPath.ToString() + @"\Data\Northwind.mdb";
    
            return objConnectionStringBuilder.ToString();
        }
    
        void LoadPage1()
        {   
            // Очищаем список, иначе будет накопление
            cbLetters.Items.Clear();
    
            // Блоки кода для читабельности, но видимость переменных не скрывают
            {   // Загружаем раскрывающийся список
                // Добавили первый элемент списка
                cbLetters.Items.Add("All");
    
                // Дополняем список буквами алфавита
                for (char ch = 'A'; ch <= 'Z'; ch++)
                {
                    cbLetters.Items.Add(ch);
                }
    
                // Устанавливаем на первый индекс 
                cbLetters.SelectedIndex = 0;
            }
    
            {   // Загружаем таблицу Customers целиком
                connection = new OleDbConnection(ConnectionString());
                adapter = new OleDbDataAdapter(
                    new OleDbCommand("SELECT * FROM Customers", connection));
                tableCustomers = new DataTable("Customers");
    
                // Добавляем (до загрузки!!!) вычислимые столбцы Tab и ViewColumn
                // для отображения одного из них в listBox
                tableCustomers.Columns.Add(new DataColumn("CustomerID", typeof(String)));
                tableCustomers.Columns.Add(new DataColumn("Tab", typeof(String)));
                tableCustomers.Columns["Tab"].DefaultValue = "\t - ";// Значение по умолчанию
                tableCustomers.Columns.Add(new DataColumn("CompanyName", typeof(String)));
                tableCustomers.Columns.Add(new DataColumn("ViewColumn", typeof(String)));
                // Компонуем столбец
                tableCustomers.Columns["ViewColumn"].Expression = "CustomerID + Tab + CompanyName";
    
                adapter.Fill(tableCustomers);// Сам откроет соединение и сам закроет
    
                // Указать первичный ключ из одного столбца для использования метода Find()
                tableCustomers.PrimaryKey =
                    new DataColumn[] { tableCustomers.Columns["CustomerID"] };
    
                // Не разрешать ввод пустых значений в поле CustomerID
                //tableCustomers.Columns["CustomerID"].AllowDBNull = false;
            }
        }
    
        // Вся таблица находится в кэше в отсоединенном режиме 
        // и поэтому обрабатывается быстро
        // Тот же самый результат можно было получить на уровне БД
        // с помощью SQL-запросов, но это намного медленнее!!!
        void cbLetters_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Еще ничего не создано или не выбрано
            // Условия проверяются слева направо до первого true
            if (cbLetters == null || cbLetters.SelectedIndex == -1
                || tableCustomers == null)
                return;
    
            // Извлекаем символ из cbLetters
            String letter = cbLetters.SelectedItem.ToString();
            if (letter == "All")
                letter = String.Empty;
    
            // Создаем для таблицы Customers объект представления и настраиваем его
            DataView view = new DataView(tableCustomers);
            view.RowFilter = "CustomerID LIKE '" + letter + "%'";
            view.Sort = "CustomerID"; 
    
            // Заполняем список ListBox столбцами CustomerID и скомпанованным
            // с учетом значения фильтра в cbLetters 
            listBox.DataSource = view;// Связываем
            listBox.DisplayMember = "ViewColumn";// Отображаем вычислимый столбец
            listBox.ValueMember = "CustomerID"; // Невидимый параметр
        }
    
        void button_DeleteClick(object sender, EventArgs e)
        {
            // Если нечего удалять
            if (cbLetters.SelectedIndex == -1 || cbLetters.Items.Count <= 0)
                return;
    
            // Ищем текущую строку в таблице по CustomerID
            Object findValue = listBox.SelectedValue;// Выделенное значение ValueMember
            DataRow row = tableCustomers.Rows.Find(findValue);
            // Помечаем строку как удаленную
            row.Delete();
        }
    
        // Отменить текущие изменения таблицы
        void button_RestoreClick(object sender, EventArgs e)
        {
            tableCustomers.RejectChanges(); ;
        }
    
        // Отправить текущие изменения таблицы в базу. Если убрать new, 
        // то будет вызываться унаследованная функция Control.Update()
        new void Update()
        {
            // Создаем построитель команды по настройкам нашего адаптера
            OleDbCommandBuilder commandBuilder = 
                new OleDbCommandBuilder(adapter);
            // Не проверять наличие чужих изменений, 
            // возможно внесенных другими со времени последней 
            // выборки данных, а записывать своей UpdateCommand поверх
            commandBuilder.ConflictOption = ConflictOption.OverwriteChanges;
            //commandBuilder.RefreshSchema();
    
            // Построитель команд сам сгенерирует нужные команды,
            // прочитав информацию в свойстве SelectCommand адаптера
            //adapter.InsertCommand = commandBuilder.GetInsertCommand();
            //adapter.UpdateCommand = commandBuilder.GetUpdateCommand();
            //adapter.DeleteCommand = commandBuilder.GetDeleteCommand();
            // Если соединение открыть, то адаптер его сам не закроет
            //connection.Open();
    
            // Адаптер сам откроет соединение, сам его закроет, 
            // сам переведет таблицу в разряд немодифицированных
            adapter.Update(tableCustomers);
    
            //adapter.InsertCommand.Connection.Close();
            // Перевести таблицу в разряд немодифицированных
            //tableCustomers.AcceptChanges();
        }
    
        // Отправить текущие изменения таблицы в базу
        void button_UpdateClick(object sender, EventArgs e)
        {
            Update();
        }
    
        DataRow newRow = null;// Проверяется обработчиками кнопок
        void button_CurrentClick(object sender, EventArgs e)
        {
            // Если ничего не выделено как текущее
            if (listBox.SelectedIndex == -1)
                return;
    
            // А не включен ли режим добавления новой строки
            // (а не испить ли нам кофею!!!)
            if (newRow != null)
            {
                MessageBox.Show("Редактируйте добавленную\n"
                    + "строку или отмените ее", "Ошибка",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
    
            // Ищем текущую строку в таблице по CustomerID
            Object findValue = listBox.SelectedValue;// Выделенное значение ValueMember
            DataRow row = tableCustomers.Rows.Find(findValue);
    
            // Заполняем текстовые поля значениями полей текущей строки
            foreach (TextBox tb in listReference)
            {
                // Имена столбцов ранее записали в свойство tb.Name;
                tb.Text = row[tb.Name].ToString();
            }
        }
    
        // Добавление новой строки
        void button_NewRowClick(object sender, EventArgs e)
        {
            if (newRow != null)
            {
                MessageBox.Show("Вы не отменили или \n"
                    + "не сохранили предыдущую строку", "Ошибка",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
    
            // Меняем цвет текстовых полей
            ChangeColor(Colors.EditColor);
    
            // Очищаем текстовые поля
            foreach (TextBox tb in listReference)
            {
                tb.Text = String.Empty;
            }
    
            // Создаем новую строку по схеме таблицы
            newRow = tableCustomers.NewRow();
        }
    
        void button_AddRowClick(object sender, EventArgs e)
        {
            if (newRow == null)
            {
                MessageBox.Show("Вы не добавили новую строку", "Ошибка",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
    
            // Заполняем поля новой строки данными из TextBox
            newRow.BeginEdit(); // Отключаем механизм проверки ограничений
            foreach (TextBox tb in listReference)
            {
                if (tb.Text == String.Empty)// Пустые строки не принимает
                    newRow[tb.Name] = System.DBNull.Value;
                else
                    newRow[tb.Name] = tb.Text;
            }
    
            if (newRow["CustomerID"].ToString() == String.Empty)
            {
                MessageBox.Show("Поле CustomerID не должно быть пустым",
                    "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
    
            // Возможно нарушение ввода, поэтому упаковываем в проверку
            try
            {
                newRow.EndEdit();// Пробуем включить проверку ограничений
                // Добавляем строку в таблицу
                tableCustomers.Rows.Add(newRow);
            }
            catch
            {
                MessageBox.Show("Поле CustomerID должно быть уникальным",
                    "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
    
            newRow = null;
    
            // Восстанавливаем цвет текстовых полей
            ChangeColor(Colors.RestoreColor);
    
            // Очищаем текстовые поля
            foreach (TextBox tb in listReference)
            {
                tb.Text = String.Empty;
            }
        }
    
        void button_ClearClick(object sender, EventArgs e)
        {
            // Обнуляем новую строку
            newRow = null;
            // Очищаем текстовые поля
            foreach (TextBox tb in listReference)
            {
                tb.Text = String.Empty;
            }
            // Восстанавливаем цвет
            ChangeColor(Colors.RestoreColor);
        }
    
        enum Colors
        {
            EditColor,
            RestoreColor
        }
    
        // Изменяем цвет текстовых полей
        void ChangeColor(Colors color)
        {
            switch (color)
            {
                case Colors.EditColor:
                    foreach (TextBox tb in listReference)
                        tb.BackColor = Color.Aqua;
                    break;
                case Colors.RestoreColor:
                    foreach (TextBox tb in listReference)
                        tb.BackColor = Color.White;
                    break;
            }
        }
    }
}
